#!/usr/bin/env python3

'''
Purge ElasticSearch Data
========================
'''

import argparse
import json
import pycurl
from datetime import date
from dateutil.relativedelta import relativedelta
from io import BytesIO
try:
    # python 3
    from urllib.parse import urlencode
except ImportError:
    # python 2
    from urllib import urlencode

parser = argparse.ArgumentParser(description='ElasticSearch Purge')
parser.add_argument('username')
parser.add_argument('password')
parser.add_argument('index')
parser.add_argument('retention')
parser.add_argument('mode')
args = parser.parse_args()


def getIndex():
    buffer = BytesIO()
    c = pycurl.Curl()
    c.setopt(c.URL, 'http://%s:%s@localhost:9200/_cat/indices?v&pretty&s=index&format=json' % (args.username, args.password))
    c.setopt(c.HTTPHEADER, ['Content-Type: application/json'])
    c.setopt(c.WRITEDATA, buffer)
    c.perform()
    c.close()
    return json.loads(buffer.getvalue())


def purgeSingle(cat, day):
    for index in cat:
        if index['index'] == args.index:
            post_data = {"query":{"bool":{"filter":{"range":{"@timestamp":{"lte":day.strftime('%Y-%m-%d')}}}}}}
            postfields = json.dumps(post_data)
            print('  Postfields: ' + postfields)
            buffer = BytesIO()
            c = pycurl.Curl()
            c.setopt(c.URL, 'http://%s:%s@localhost:9200/%s/_delete_by_query' % (args.username, args.password, args.index))
            c.setopt(c.HTTPHEADER, ['Content-Type: application/json'])
            c.setopt(c.WRITEDATA, buffer)
            c.setopt(c.POSTFIELDS, postfields)
            c.perform()
            c.close()
            print('  Delete: ' + buffer.getvalue().decode('utf8'))

            buffer = BytesIO()
            c = pycurl.Curl()
            c.setopt(c.URL, 'http://%s:%s@localhost:9200/%s/_forcemerge?only_expunge_deletes=true' % (args.username, args.password, args.index))
            c.setopt(c.HTTPHEADER, ['Content-Type: application/json'])
            c.setopt(c.WRITEDATA, buffer)
            c.setopt(c.POSTFIELDS, '')
            c.perform()
            c.close()
            print('  Purge: ' + buffer.getvalue().decode('utf8'))
            return

    print('Index not found')


def purgeDaily(cat, day):
    prefix = args.index + '-'
    range = prefix + day.strftime('%Y.%m.%d')
    for index in cat:
        if index['index'].find(prefix) == 0 and index['index'] < range:
            print('  Index: ' + index['index'])
            buffer = BytesIO()
            c = pycurl.Curl()
            c.setopt(c.URL, 'http://%s:%s@localhost:9200/%s' % (args.username, args.password, index['index']))
            c.setopt(c.HTTPHEADER, ['Content-Type: application/json'])
            c.setopt(c.WRITEDATA, buffer)
            c.setopt(c.CUSTOMREQUEST, 'DELETE')
            c.perform()
            c.close()
            print('  Delete: ' + buffer.getvalue().decode('utf8'))


cat = getIndex()
day = date.today() + relativedelta(days=-int(args.retention))
print('Purge index %s older than %s' % (args.index, day))

if args.mode == 'single':
    purgeSingle(cat, day)
elif args.mode == 'daily':
    purgeDaily(cat, day)
else:
    print('Unknown mode')
